import { useSiteContext } from '@context/SiteContext'
import useAuth from '@hooks/useAuth'
import useLanguage from '@hooks/useLanguage'
import { A } from '@solidjs/router'
import { IconDashboard, IconLogout, IconTriangle } from '@tabler/icons-solidjs'
import { For, Show, mergeProps } from 'solid-js'
import { Dynamic } from 'solid-js/web'
import './navbar.scss'

const language = useLanguage()

export const NAV_ITEM = (admin = false) => [
  {
    path: '/dashboard',
    show: admin,
    icon: IconDashboard,
    text: language?.ui.dashboard,
  },
  {
    show: true,
    icon: IconTriangle,
    text: 'API',
    children: [
      {
        show: true,
        icon: IconDashboard,
        text: 'Test',
        children: [
          {
            path: '/dashboard',
            show: true,
            icon: IconDashboard,
            text: language?.ui.dashboard,
          },
          {
            path: '/test',
            show: true,
            icon: IconDashboard,
            text: 'Test',
          },
        ],
      },
      {
        path: '/api',
        show: true,
        icon: IconDashboard,
        text: 'Get',
      },
    ],
  },
  {
    path: '/ware-house',
    show: true,
    icon: IconDashboard,
    text: language?.ui.houses,
  },
]

function NavbarItem(props) {
  const merged = mergeProps({ active: true }, props)
  return (
    <Show
      when={merged.active && merged.path}
      fallback={
        <>
          <Dynamic component={merged.icon} />
          {merged.text}
        </>
      }
    >
      <A href={merged.path}>
        <Dynamic component={merged.icon} />
        {merged.text}
      </A>
    </Show>
  )
}

function NavbarWithChildren(props) {
  return (
    <li class="mb-2">
      <h2 class="menu-title flex items-center gap-2">
        <NavbarItem {...props} active={false} />
      </h2>
      <ul class="pl-4">
        <For each={props.children}>
          {(child) => {
            if (!child.show) return
            if (child.children) {
              return <NavbarWithChildren {...child} />
            } else {
              return (
                <li class="mb-2">
                  <NavbarItem {...child} />
                </li>
              )
            }
          }}
        </For>
      </ul>
    </li>
  )
}

export default function Navbar() {
  const { store, setAuth } = useSiteContext()
  const { clickLogOut } = useAuth(setAuth)

  const logOut = async () => {
    try {
      await clickLogOut()
    } catch (error) {
      console.log({
        status: 'danger',
        title: 'Logout fail!',
        closable: false,
      })
    }
  }

  return (
    <div class="drawer-side lg:h-[calc(100svh-64px)]">
      <label for="nav-menu" aria-label="close sidebar" class="drawer-overlay" />
      <div class="bg-base-200 w-80 min-h-full">
        <Show when={store.auth}>
          <div class="flex items-center justify-between px-3 pt-5 lg:hidden">
            <div class="avatar">
              <div class="w-9 mask mask-hexagon">
                <img
                  src={`https://ui-avatars.com/api/?name=${store.userInfo?.name}`}
                  alt="avatar"
                />
              </div>
            </div>
            <span class="mx-3 line-clamp-1">
              <A href="/me">{store.userInfo?.name}</A>
            </span>
            <button class="btn btn-ghost btn-sm" onClick={logOut}>
              <IconLogout size={16} />
            </button>
          </div>
          <div class="divider divider-success mb-0 lg:hidden">
            <IconTriangle size={30} />
          </div>
        </Show>
        <ul class="menu w-80 text-base-content pt-3 px-3 pb-6">
          <For each={NAV_ITEM(store?.userInfo?.isAdmin)}>
            {(item) => {
              if (!item.show) return
              if (item.children) {
                return <NavbarWithChildren {...item} />
              } else {
                return (
                  <li class="mb-2">
                    <NavbarItem {...item} />
                  </li>
                )
              }
            }}
          </For>
        </ul>
      </div>
    </div>
  )
}
